import { defineConfig, mergeConfig } from 'vite';
import vuePlugin from '@vitejs/plugin-vue';
import viteJsxPlugin from '@vitejs/plugin-vue-jsx';

async function startNuxtAndGetViteConfig(rootDir = process.cwd()) {
  const { loadNuxt, buildNuxt } = await import('@nuxt/kit');
  const nuxt = await loadNuxt({
    cwd: rootDir,
    dev: false,
    overrides: {
      ssr: false,
      app: {
        rootId: "nuxt-test"
      }
    }
  });
  if (!nuxt.options._installedModules.find((i) => i?.meta?.name === "nuxt-vitest")) {
    throw new Error(
      "Failed to load nuxt-vitest module. You may need to add it to your nuxt.config."
    );
  }
  const promise = new Promise((resolve, reject) => {
    nuxt.hook("vite:extendConfig", (viteConfig, { isClient }) => {
      if (isClient) {
        resolve({ nuxt, viteConfig });
        throw new Error("_stop_");
      }
    });
    buildNuxt(nuxt).catch((err) => {
      if (!err.toString().includes("_stop_")) {
        reject(err);
      }
    });
  }).finally(() => nuxt.close());
  return promise;
}
const vuePlugins = {
  "vite:vue": [vuePlugin, "vue"],
  "vite:vue-jsx": [viteJsxPlugin, "vueJsx"]
};
async function getVitestConfigFromNuxt(options) {
  if (!options)
    options = await startNuxtAndGetViteConfig();
  options.viteConfig.plugins = options.viteConfig.plugins || [];
  options.viteConfig.plugins = options.viteConfig.plugins.filter((p) => p?.name !== "nuxt:import-protection");
  for (const name in vuePlugins) {
    if (!options.viteConfig.plugins?.some((p) => p?.name === name)) {
      const [plugin, key] = vuePlugins[name];
      options.viteConfig.plugins.push(plugin(options.viteConfig[key]));
    }
  }
  return {
    ...options.viteConfig,
    server: {
      ...options.viteConfig.server,
      middlewareMode: false
    },
    test: {
      ...options.viteConfig.test,
      dir: options.nuxt.options.rootDir,
      environmentOptions: {
        ...options.viteConfig.test?.environmentOptions,
        nuxtRuntimeConfig: options.nuxt.options.runtimeConfig
      },
      environmentMatchGlobs: [
        ["**/*.nuxt.{test,spec}.{js,mjs,cjs,ts,mts,cts,jsx,tsx}", "nuxt"],
        ["{test,tests}/nuxt/**.*", "nuxt"],
        ...options.viteConfig.test?.environmentMatchGlobs || []
      ],
      deps: {
        ...options.viteConfig.test?.deps,
        inline: options.viteConfig.test?.deps?.inline === true ? true : [
          // vite-node defaults
          /\/(nuxt|nuxt3)\//,
          /^#/,
          "vite",
          // additional deps
          "vue",
          "vitest-environment-nuxt",
          ...options.nuxt.options.build.transpile.filter(
            (r) => typeof r === "string" || r instanceof RegExp
          ),
          ...options.viteConfig.test?.deps?.inline || []
        ]
      }
    }
  };
}
function defineVitestConfig(config = {}) {
  return defineConfig(async () => {
    if (process.env.__NUXT_VITEST_RESOLVED__)
      return config;
    return mergeConfig(await getVitestConfigFromNuxt(), config);
  });
}

export { defineVitestConfig, getVitestConfigFromNuxt };
