import { defineNuxtModule, installModule, resolvePath, logger } from '@nuxt/kit';
import { mergeConfig } from 'vite';
import { getVitestConfigFromNuxt } from './config.mjs';
import { getPort } from 'get-port-please';
import { h } from 'vue';
import { debounce } from 'perfect-debounce';
import { isCI } from 'std-env';
import '@vitejs/plugin-vue';
import '@vitejs/plugin-vue-jsx';

const vitePluginBlocklist = ["vite-plugin-vue-inspector", "vite-plugin-inspect"];
const module = defineNuxtModule({
  meta: {
    name: "nuxt-vitest",
    configKey: "vitest"
  },
  defaults: {
    startOnBoot: false,
    logToConsole: false
  },
  async setup(options, nuxt) {
    await installModule("vitest-environment-nuxt/module");
    if (!nuxt.options.dev)
      return;
    if (process.env.TEST || process.env.VITE_TEST)
      return;
    const rawViteConfigPromise = new Promise((resolve) => {
      nuxt.hook("app:resolve", () => {
        nuxt.hook("vite:extendConfig", (config, { isClient }) => {
          if (isClient)
            resolve(config);
        });
      });
    });
    const PORT = await getPort({ port: 15555 });
    const URL = `http://localhost:${PORT}/__vitest__/`;
    let loaded = false;
    let promise;
    let ctx = void 0;
    let testFiles = null;
    const updateTabs = debounce(() => {
      nuxt.callHook("devtools:customTabs:refresh");
    }, 100);
    async function start() {
      const rawViteConfig = mergeConfig({}, await rawViteConfigPromise);
      const viteConfig = mergeConfig(
        await getVitestConfigFromNuxt({ nuxt, viteConfig: rawViteConfig }),
        {
          server: {
            middlewareMode: false
          }
        }
      );
      viteConfig.plugins = (viteConfig.plugins || []).filter((p) => {
        return !vitePluginBlocklist.includes(p?.name);
      });
      process.env.__NUXT_VITEST_RESOLVED__ = "true";
      const { startVitest } = await import(await resolvePath("vitest/node"));
      const customReporter = {
        onInit(_ctx) {
          ctx = _ctx;
        },
        onTaskUpdate() {
          testFiles = ctx.state.getFiles();
          updateTabs();
        },
        onFinished() {
          testFiles = ctx.state.getFiles();
          updateTabs();
        }
      };
      const watchMode = !process.env.NUXT_VITEST_DEV_TEST && !isCI;
      const vitestConfig = watchMode ? {
        passWithNoTests: true,
        ...options.vitestConfig,
        reporters: options.logToConsole ? [
          ...toArray(options.vitestConfig?.reporters ?? ["default"]),
          customReporter
        ] : [customReporter],
        // do not report to console
        watch: true,
        ui: true,
        open: false,
        api: {
          port: PORT
        }
      } : {
        ...options.vitestConfig,
        watch: false
      };
      viteConfig.configFile = false;
      const promise2 = startVitest("test", [], vitestConfig, viteConfig);
      promise2.catch(() => process.exit(1));
      if (watchMode) {
        logger.info(`Vitest UI starting on ${URL}`);
        await new Promise((resolve) => setTimeout(resolve, 1e3));
      } else {
        promise2.then((v) => v?.close()).then(() => process.exit());
        promise2.catch(() => process.exit(1));
      }
      loaded = true;
    }
    nuxt.hook("devtools:customTabs", (tabs) => {
      const failedCount = testFiles?.filter((f) => f.result?.state === "fail").length ?? 0;
      const passedCount = testFiles?.filter((f) => f.result?.state === "pass").length ?? 0;
      const totalCount = testFiles?.length ?? 0;
      tabs.push({
        title: "Vitest",
        name: "vitest",
        icon: "logos-vitest",
        view: loaded ? {
          type: "iframe",
          src: URL
        } : {
          type: "launch",
          description: "Start tests along with Nuxt",
          actions: [
            {
              label: promise ? "Starting..." : "Start Vitest",
              pending: !!promise,
              handle: () => {
                promise = promise || start();
                return promise;
              }
            }
          ]
        },
        extraTabVNode: totalCount ? h("div", { style: { color: failedCount ? "orange" : "green" } }, [
          h("span", {}, passedCount),
          h("span", { style: { opacity: "0.5", fontSize: "0.9em" } }, "/"),
          h(
            "span",
            { style: { opacity: "0.8", fontSize: "0.9em" } },
            totalCount
          )
        ]) : void 0
      });
    });
    if (options.startOnBoot) {
      promise = promise || start();
      promise.then(updateTabs);
    }
  }
});
function toArray(value) {
  return Array.isArray(value) ? value : [value];
}

export { module as default };
